@@ -15,6 +15,32 @@ function(x,...) {
   return(xx)
 }
 
+`xcoredata.default` <-
+function(x,...) {
+  x.attr <- attributes(x)
+  original.attr <- x.attr[!names(x.attr) %in%
+                          c('dim','dimnames')]
+  original.attr
+}
+
+
+`xcoredata` <-
+function(x,...) {
+  UseMethod('xcoredata')
+}
+
+`xcoredata<-` <- function(x,value) {
+  UseMethod('xcoredata<-')
+}
+
+`xcoredata<-.default` <- function(x,value) {
+  for(att in names(value)) {
+    if(!att %in% c('dim','dimnames'))
+      attr(x,att) <- value[[att]]
+  }
+  x
+}
+
 `xtsAttributes` <-
 function(x, user=NULL) {
   # get all additional attributes not standard to xts object
