@@ -58,35 +58,41 @@ function(x, i, j, drop = TRUE, ...)
     POSIXindex <- index(x)
 
     if (missing(i)) 
-        i <- 1:NROW(x)
+      i <- 1:NROW(x)
+    if (timeBased(i)) 
+      i <- as.character(as.POSIXct(i)) 
     if (is.character(i)) {
       # enables subsetting by date style strings
       # must be able to process - and then allow for operations???
 
-      if(!identical(grep("::",i),integer(0))) {
-        # range operator
-        dates <- strsplit(i,'::')[[1]]
+      i.tmp <- NULL
+      for(ii in i) {
+        if(!identical(grep("::",ii),integer(0))) {
+          # range operator
+          dates <- strsplit(ii,'::')[[1]]
+          
+          # test for single side range operation
+          first.time <- ifelse(dates[1]=="",
+                               POSIXindex[1],
+                               do.call('firstof',
+                                        as.list(as.numeric(strsplit(dates[1],':|-|/| ')[[1]]))))
+          last.time <- ifelse(length(dates)==1,
+                               POSIXindex[length(POSIXindex)],
+                               do.call('lastof',
+                                        as.list(as.numeric(strsplit(dates[2],':|-|/| ')[[1]]))))
+        } else {
+          # if single date is given - get start and end points if resolution of
+          # series is greater than the time specified
+          dates <- ii
+          first.time <- do.call('firstof',
+                                as.list(as.numeric(strsplit(dates,':|-|/| ')[[1]])))
+          last.time <- do.call('lastof',
+                                as.list(as.numeric(strsplit(dates,':|-|/| ')[[1]])))
+        }      
         
-        # test for single side range operation
-        first.time <- ifelse(dates[1]=="",
-                             POSIXindex[1],
-                             do.call('firstof',
-                                      as.list(as.numeric(strsplit(dates[1],':|-|/| ')[[1]]))))
-        last.time <- ifelse(length(dates)==1,
-                             POSIXindex[length(POSIXindex)],
-                             do.call('lastof',
-                                      as.list(as.numeric(strsplit(dates[2],':|-|/| ')[[1]]))))
-      } else {
-        # if single date is given - get start and end points if resolution of
-        # series is greater than the time specified
-        dates <- i
-        first.time <- do.call('firstof',
-                              as.list(as.numeric(strsplit(dates,':|-|/| ')[[1]])))
-        last.time <- do.call('lastof',
-                              as.list(as.numeric(strsplit(dates,':|-|/| ')[[1]])))
-      }      
-      
-      i <- which(POSIXindex <= last.time & POSIXindex >= first.time)
+        i.tmp <- c(i.tmp,which(POSIXindex <= last.time & POSIXindex >= first.time))
+      }
+      i <- i.tmp
     }
 
     class(x) <- "zoo"
@@ -155,28 +161,55 @@ function(..., deparse.level=1) {
  args <- list(...)
 
  # Store original class attributes
- xts.ROWNAMES <- sapply(args, function(x) attr(x, ".ROWNAMES"))
  xts.CLASS <- sapply(args, CLASS)
- xts.CLASSattr <- sapply(args, xtsAttributes, user=FALSE)
- xts.USERattr <- unlist(sapply(args, xtsAttributes, user=TRUE))
+ xts.CLASSattr <- lapply(args, xtsAttributes, user=FALSE)
+ xts.USERattr <- lapply(args, xtsAttributes, user=TRUE)
+ has.ROWNAMES <- any( lapply(args, function(x) any(names(attributes(x))=='.ROWNAMES')) )
 
+ # Ensure index attributes match
+ # store as POSIXct, convert to class of first object index
+ iclass <- class( index(args[[1]]) )
+ timeBasedClass <- sapply( iclass, function(x) xts:::timeBased(structure(1,class=x)) )
+ iclass <- iclass[timeBasedClass][1]
+ for(arg in args) {
+   index(arg) <- as.POSIXct(index(arg))
+ }
+ 
  # Bind objects
  ret <- zoo:::rbind.zoo(...)
  ret <- structure( ret, class=c('xts','zoo') )
 
- # Drop CLASS attribute if _not_ the same for all objects
- xts.CLASS.eq <- sapply(xts.CLASS, function(x) identical(x,xts.CLASS[[1]]))
- if( all(xts.CLASS.eq) ) {
-   CLASS(ret) <- xts.CLASS[[1]]
+ # Restore first object index class
+ index(ret) <- do.call( paste('as', iclass, sep='.'), list(index(ret)) )
+
+ # Drop CLASS & USER attributes if they are not the same for all objects
+ all.CLASS <- all( sapply(xts.CLASS, function(x) identical(x,xts.CLASS[[1]])) )
+ all.USER <- all( sapply(xts.USERattr, function(x) identical(x,xts.USERattr[[1]])) )
+ 
+ if( all.CLASS ) {
+   # Need a better way to deal with different xtsAttributes than
+   # simply assigning them to the value of the first object.
    xts.CLASSattr <- xts.CLASSattr[[1]]
+   CLASS(ret) <- xts.CLASS[[1]]
+   attr(ret, '.CLASSnames') <- attr(args[[1]], '.CLASSnames')
+
  } else {
    CLASS(ret) <- NULL
    xts.CLASSattr <- NULL
  }
-
+ if( has.ROWNAMES ) {
+ # .ROWNAMES attribute may not be similar, so set .ROWNAMES to index for now,
+ # if it exists in at least one object
+   attr(ret, '.ROWNAMES') <- as.character(index(ret))
+ } else {
+   attr(ret, '.ROWNAMES') <- NULL
+ }
+ if( all.USER ) {
+   xts.USERattr <- xts.USERattr[[1]]
+ } else {
+   xts.USERattr <- NULL
+ }
  # Re-attach _xts_ attributes
- # Need a better way to deal with different xtsAttributes than
- # simply assigning them to the value of the first object...
  xtsAttributes(ret) <- c( xts.CLASSattr, xts.USERattr )
 
  return(ret)
@@ -184,32 +217,50 @@ function(..., deparse.level=1) {
 
 `cbind.xts` <-
 function(..., deparse.level=1) {
+
  args <- list(...)
 
  # Store original class attributes
- xts.ROWNAMES <- sapply(args, function(x) attr(x, ".ROWNAMES"))
  xts.CLASS <- sapply(args, CLASS)
- xts.CLASSattr <- sapply(args, xtsAttributes, user=FALSE)
- xts.USERattr <- unlist(sapply(args, xtsAttributes, user=TRUE))
+ xts.CLASSattr <- lapply(args, xtsAttributes, user=FALSE)
+ xts.USERattr <- lapply(args, xtsAttributes, user=TRUE)
+ has.ROWNAMES <- any( lapply(args, function(x) any(names(attributes(x))=='.ROWNAMES')) )
 
  # Bind objects
  ret <- zoo:::cbind.zoo(...)
  ret <- structure( ret, class=c('xts','zoo') )
 
- # Drop CLASS attribute if _not_ the same for all objects
- xts.CLASS.eq <- sapply(xts.CLASS, function(x) identical(x,xts.CLASS[[1]]))
- if( all(xts.CLASS.eq) ) {
-   CLASS(ret) <- xts.CLASS[[1]]
+ # Drop CLASS & USER attributes if they are not the same for all objects
+ all.CLASS <- all( sapply(xts.CLASS, function(x) identical(x,xts.CLASS[[1]])) )
+ all.USER <- all( sapply(xts.USERattr, function(x) identical(x,xts.USERattr[[1]])) )
+ 
+ if( all.CLASS ) {
+   # Need a better way to deal with different xtsAttributes than
+   # simply assigning them to the value of the first object.
    xts.CLASSattr <- xts.CLASSattr[[1]]
+   CLASS(ret) <- xts.CLASS[[1]]
+   attr(ret, '.CLASSnames') <- attr(args[[1]], '.CLASSnames')
+
  } else {
    CLASS(ret) <- NULL
    xts.CLASSattr <- NULL
  }
-
+ if( has.ROWNAMES ) {
+ # .ROWNAMES attribute may not be similar, so set .ROWNAMES to index for now,
+ # if it exists in at least one object.
+   attr(ret, '.ROWNAMES') <- as.character(index(ret))
+ } else {
+   attr(ret, '.ROWNAMES') <- NULL
+ }
+ if( all.USER ) {
+   xts.USERattr <- xts.USERattr[[1]]
+ } else {
+   xts.USERattr <- NULL
+ }
  # Re-attach _xts_ attributes
- # Need a better way to deal with different xtsAttributes than
- # simply assigning them to the value of the first object...
  xtsAttributes(ret) <- c( xts.CLASSattr, xts.USERattr )
+ # Add rownames
+ rownames(ret) <- as.character(index(ret))
 
  return(ret)
 }
@@ -218,4 +269,3 @@ function(..., deparse.level=1) {
 function(...) {
   rbind.xts(...)
 }
-
